﻿Imports System.Threading
Imports System.Threading.Tasks
Imports System.Windows.Forms

Namespace SolSoft.ObjectOrientedGuiStartup.WindowsForms
	''' <summary>
	''' Fifth refactoring, with centralized exception handling
	''' </summary>
	Friend Class Program5a
		''' <summary>
		''' The main entry point for the application.
		''' </summary>
		<STAThread>
		Shared Sub Main()
			Application.EnableVisualStyles()
			Application.SetCompatibleTextRenderingDefault(False)
			SynchronizationContext.SetSynchronizationContext(New WindowsFormsSynchronizationContext())

			AddHandler Application.ThreadException, AddressOf Application_ThreadException
			AddHandler AppDomain.CurrentDomain.UnhandledException, AddressOf CurrentDomain_UnhandledException
			AddHandler TaskScheduler.UnobservedTaskException, AddressOf TaskScheduler_UnobservedTaskException


			Dim p As New Program5a()
			AddHandler p.ExitRequested, AddressOf p_ExitRequested

			Dim programStart As Task = p.StartAsync()
			HandleExceptions(programStart)

			Application.Run()
		End Sub

		Private Shared async Sub HandleExceptions(ByVal task As Task)
			Try
				await Task.Yield() 'ensure this runs as a continuation
                Await task
			Catch ex As Exception
				'deal with exception, either with message box
				'or delegating to general exception handling logic you may have wired up 
				'e.g. to Application.ThreadException and AppDomain.UnhandledException
				MessageBox.Show(ex.ToString())

				Application.Exit()
			End Try
		End Sub

		Private Shared Sub p_ExitRequested(ByVal sender As Object, ByVal e As EventArgs)
			Application.ExitThread()
		End Sub


		Private Shared Sub TaskScheduler_UnobservedTaskException(ByVal sender As Object, ByVal e As UnobservedTaskExceptionEventArgs)
			HandleException(e.Exception, False)
			e.SetObserved()
		End Sub

		Private Shared Sub CurrentDomain_UnhandledException(ByVal sender As Object, ByVal e As UnhandledExceptionEventArgs)
			Dim exception As Exception = TryCast(e.ExceptionObject, Exception)
			If exception Is Nothing Then
				If e.ExceptionObject IsNot Nothing Then
					exception = New Exception(e.ExceptionObject.ToString())
				Else
					exception = New Exception()
				End If
			End If

			HandleException(exception, False)
		End Sub

		Private Shared Sub Application_ThreadException(ByVal sender As Object, ByVal e As ThreadExceptionEventArgs)
			HandleException(e.Exception, False)
		End Sub

		Private Shared Sub HandleException(ByVal exception As Exception, ByVal quit As Boolean)
			MessageBox.Show(exception.ToString())

			If quit Then
				Application.Exit()
			End If
		End Sub


		Private m_mainForm As Form1
		Private Sub New()
			m_mainForm = New Form1()
			AddHandler m_mainForm.FormClosed, AddressOf m_mainForm_FormClosed
		End Sub

		Public async Function StartAsync() As Task
			await m_mainForm.InitializeAsync()

			m_mainForm.Show()
		End Function

		Public Event ExitRequested As EventHandler(Of EventArgs)
		Private Sub m_mainForm_FormClosed(ByVal sender As Object, ByVal e As FormClosedEventArgs)
			OnExitRequested(EventArgs.Empty)
		End Sub

		Protected Overridable Sub OnExitRequested(ByVal e As EventArgs)
			RaiseEvent ExitRequested(Me, e)
		End Sub
	End Class
End Namespace
