﻿Imports System.Text
Imports System.Threading
Imports System.Threading.Tasks
Imports System.Windows.Threading

Namespace SolSoft.ObjectOrientedGuiStartup.Wpf
	''' <summary>
	''' Asynchronous with splash screen
	''' </summary>
	Friend Class Program6
		''' <summary>
		''' The main entry point for the application.
		''' </summary>
		<STAThread>
		Shared Sub Main()
			Dim app As AppNoStartupUri = New AppNoStartupUri With {.ShutdownMode = ShutdownMode.OnExplicitShutdown}
			app.InitializeComponent()
			SynchronizationContext.SetSynchronizationContext(New DispatcherSynchronizationContext())

			Dim p As New Program6()
			AddHandler p.ExitRequested, Sub(sender, e) app.Shutdown()

			Dim programStart As Task = p.StartAsync()
			HandleExceptions(programStart, app)

			app.Run()
		End Sub

		Private Shared async Sub HandleExceptions(ByVal task As Task, ByVal wpfApplication As Application)
			Try
				await Task.Yield() 'ensure this runs as a continuation
                Await task
			Catch ex As Exception
				'deal with exception, either with message box
				'or delegating to general exception handling logic you may have wired up 
				'e.g. to app.DispatcherUnhandledException and AppDomain.UnhandledException
				MessageBox.Show(ex.ToString())

				wpfApplication.Shutdown()
			End Try
		End Sub

		Private Sub New()
		End Sub

		Public async Function StartAsync() As Task
			Dim viewModel As New MainViewModel()
			AddHandler viewModel.CloseRequested, AddressOf viewModel_CloseRequested

			Dim windowClosed As EventHandler = Sub(sender, e) viewModel.RequestClose()

			Dim splashScreen As New SplashScreen() 'not disposable, but I'm keeping the same structure
				AddHandler splashScreen.Closed, windowClosed 'if user closes splash screen, let's quit
				splashScreen.Show()

				await viewModel.InitializeAsync()

				Dim mainForm As New MainWindow()
				AddHandler mainForm.Closed, windowClosed
				mainForm.DataContext = viewModel
				mainForm.Show()

				splashScreen.Owner = mainForm
				RemoveHandler splashScreen.Closed, windowClosed
				splashScreen.Close()
		End Function

		Public Event ExitRequested As EventHandler(Of EventArgs)
		Private Sub viewModel_CloseRequested(ByVal sender As Object, ByVal e As EventArgs)
			OnExitRequested(EventArgs.Empty)
		End Sub

		Protected Overridable Sub OnExitRequested(ByVal e As EventArgs)
			RaiseEvent ExitRequested(Me, e)
		End Sub
	End Class
End Namespace
